localparam  IDLE             = 0; 
localparam  HEAD             = 1; 
localparam  DATS             = 2; 
    
(*keep="true"*)
reg [DATS:IDLE]    ST;
reg [DATS:IDLE]   nST;

//synthesis translate_off
reg [63:0] ST_STRING;
always @(*)
  case(1'b1)
    ST[IDLE]      : ST_STRING = "IDLE"; 
    ST[HEAD]      : ST_STRING = "HEAD"; 
    ST[DATS]      : ST_STRING = "DATS";
    default       : ST_STRING = "    ";
  endcase
//synthesis translate_on

always@(posedge clk)
if( rst )       ST <= 'd1;
else            ST <= nST;

always@*
begin
  nST = 'd0;
  case(1'b1)
    ST[IDLE]:
      if( axis_tvalid  )        nST[HEAD] = 1'b1;
      else                      nST[IDLE] = 1'b1;
    ST[HEAD]:
      if( axis_tvalid & axis_tready & axis_tlast  )     nST[IDLE]  = 1'b1;
      else if( axis_tvalid & axis_tready & pktOver  )   nST[DATS]  = 1'b1;
      else                                              nST[HEAD]  = 1'b1;
      
    ST[DATS]:
      if( axis_tvalid & axis_tready & axis_tlast  )     nST[IDLE]  = 1'b1;
      else                                              nST[DATS]  = 1'b1;
      
    default:                                            nST[IDLE]  = 1'b1;
  endcase
end

assign axis_tready =  ST[HEAD] | ST[DATS] ;

reg [7:0]  pcnt; 

always@(posedge clk)
if( rst )                           pcnt <= 'd5;
else if( ST[IDLE] & nST[HEAD] )     pcnt <= 'h5;
else if( ST[HEAD] & axis_tvalid & axis_tready ) pcnt <= pcnt - 1;

assign pktOver = pcnt ==1 ;



localparam    HR_IDLE            = 'd00;
localparam    HR_Reset           = 'd01;
localparam    HR_AwaitCOMINIT    = 'd02;
localparam    HR_AwaitNoCOMINIT  = 'd03;
localparam    HR_Calibrate       = 'd04;
localparam    HR_COMWAKE         = 'd05;
localparam    HR_AwaitCOMWAKE    = 'd06;
localparam    HR_AwaitNoCOMWAKE  = 'd07;
localparam    HR_AwaitAlign      = 'd08;
localparam    HR_SendAlign       = 'd09;
localparam    HR_Ready           = 'd10;
localparam    HR_Partial         = 'd11;
localparam    HR_Slumber         = 'd12;
localparam    HR_AdjustSpeed     = 'd13;

(*keep="true"*)
reg [HR_AdjustSpeed: HR_IDLE]    ST  =1;
reg [HR_AdjustSpeed: HR_IDLE]   nST  =1;


always@*
begin
  nST = 'h0;
  case(1'b1)    
    ST[HR_IDLE]:
      if( phy_ready  )                    nST[HR_Reset]           = 1'b1;
      else                                nST[HR_IDLE]            = 1'b1;
    
    ST[HR_Reset]:
      if( txcomfinish )                   nST[HR_AwaitCOMINIT]    = 1'b1;
      else                                nST[HR_Reset]           = 1'b1;
    ST[HR_AwaitCOMINIT]:                                          
      if( rxcominitdet )                  nST[HR_AwaitNoCOMINIT]  = 1'b1;
      else if(  timeCountDone )           nST[HR_Reset]           = 1'b1;
      else                                nST[HR_AwaitCOMINIT]    = 1'b1;
      
    ST[HR_AwaitNoCOMINIT]:                                        
      if( rxcominitdet )                  nST[HR_AwaitNoCOMINIT]  = 1'b1;
      else                                nST[HR_Calibrate]       = 1'b1;
        
    ST[HR_Calibrate]: 
      if( timeCountDone )                 nST[HR_COMWAKE]         = 1'b1;
      else                                nST[HR_Calibrate]       = 1'b1;
    
    ST[HR_COMWAKE]:                                         
      if( rxcomwakedet )                  nST[HR_AwaitNoCOMWAKE]  = 1'b1;
      else if( txcomfinish )              nST[HR_AwaitCOMWAKE]    = 1'b1;
      else                                nST[HR_COMWAKE]         = 1'b1;

    ST[HR_AwaitCOMWAKE]:                                        
      if( rxcomwakedet )                  nST[HR_AwaitNoCOMWAKE]  = 1'b1;
      else if( timeCountDone )            nST[HR_Reset]           = 1'b1;
      //else if( timeCountDone )            nST[HR_COMWAKE]         = 1'b1;
      else                                nST[HR_AwaitCOMWAKE]    = 1'b1;
    
    ST[HR_AwaitNoCOMWAKE]:                                        
      if( rxcomwakedet )                  nST[HR_AwaitNoCOMWAKE]  = 1'b1;
      else                                nST[HR_AwaitAlign]      = 1'b1;
    
    ST[HR_AwaitAlign]:                                        
      //if( aligndet )                      nST[HR_AdjustSpeed]     = 1'b1;
      if( aligndet )                      nST[HR_SendAlign]     = 1'b1;
      else if( timeCountDone )            nST[HR_Reset]           = 1'b1;
      else                                nST[HR_AwaitAlign]      = 1'b1;
    
    ST[HR_SendAlign]:
      if( three_NonAlign_Det  )           nST[HR_Ready]           = 1'b1;
      else                                nST[HR_SendAlign]       = 1'b1;
      
    ST[HR_Ready]:
      if( Partial )                       nST[HR_Partial]         = 1'b1;
      else if( Slumber )                  nST[HR_Slumber]         = 1'b1;
      else if(  rxcominitdet )            nST[HR_IDLE]            = 1'b1;
     //else if()   nST[HR_Reset]   =   1'b1;
      else                                nST[HR_Ready]           =   1'b1;
    ST[HR_Partial]:
      if( !Partial & !rxcomwakedet)       nST[HR_COMWAKE]         =   1'b1;
      else if( !Partial & rxcomwakedet)   nST[HR_AwaitNoCOMWAKE]  =   1'b1;
      else if( Slumber & Automatic_Partial_to_Slumber_Support )
                                          nST[HR_Slumber]         =   1'b1;
      else                                nST[HR_COMWAKE]         =   1'b1;
    ST[HR_Slumber]:
      if( !Partial & !rxcomwakedet)       nST[HR_COMWAKE]         =   1'b1;
      else if( !Partial & rxcomwakedet)   nST[HR_AwaitNoCOMWAKE]  =   1'b1;
      else                                nST[HR_Slumber]         =   1'b1;
      
    ST[HR_Slumber]:
      if( !Partial & !rxcomwakedet)       nST[HR_COMWAKE]         =   1'b1;
      else if( !Partial & rxcomwakedet)   nST[HR_AwaitNoCOMWAKE]  =   1'b1;
      else                                nST[HR_Slumber]         =   1'b1;
    
    ST[HR_AdjustSpeed]:
      if( AdjustSpeedComplet   )          nST[HR_SendAlign]       =   1'b1;
      else                                nST[HR_AdjustSpeed]     =   1'b1;
      default:                            nST[HR_IDLE]           =   1'b1;
    endcase    
end
